@@ -388,6 +388,7 @@ populate_adv_dialog (LibreswanEditor *self)
 	populate_widget (self, "phase2_lifetime_entry", NM_LIBRESWAN_KEY_SALIFETIME, NULL, NULL);
 	populate_widget (self, "rekey_checkbutton", NM_LIBRESWAN_KEY_REKEY, NULL, "no");
 	populate_widget (self, "pfs_checkbutton", NM_LIBRESWAN_KEY_PFS, NULL, "no");
+	populate_widget (self, "local_network_entry", NM_LIBRESWAN_KEY_LOCALNETWORK, NULL, NULL);
 	populate_widget (self, "remote_network_entry", NM_LIBRESWAN_KEY_REMOTENETWORK, NULL, NULL);
 	populate_widget (self, "narrowing_checkbutton", NM_LIBRESWAN_KEY_NARROWING, NULL, "yes");
 	populate_widget (self, "fragmentation_combo", NM_LIBRESWAN_KEY_FRAGMENTATION, NULL, "force");
@@ -397,6 +398,8 @@ populate_adv_dialog (LibreswanEditor *self)
 	populate_widget (self, "dpd_action_combo", NM_LIBRESWAN_KEY_DPDACTION, NULL, NULL);
 	populate_widget (self, "ipsec_interface_entry", NM_LIBRESWAN_KEY_IPSEC_INTERFACE, NULL, NULL);
 	populate_widget (self, "authby_entry", NM_LIBRESWAN_KEY_AUTHBY, NULL, NULL);
+	populate_widget (self, "disable_modecfgclient_checkbutton", NM_LIBRESWAN_KEY_LEFTMODECFGCLIENT, NULL, "no");
+	populate_widget (self, "remote_cert_entry", NM_LIBRESWAN_KEY_RIGHTCERT, NULL, NULL);
 }
 
 static gboolean
@@ -483,6 +486,7 @@ init_editor_plugin (LibreswanEditor *self,
 	hook_stuff_changed_cb (self, "phase2_lifetime_entry");
 	hook_stuff_changed_cb (self, "rekey_checkbutton");
 	hook_stuff_changed_cb (self, "pfs_checkbutton");
+	hook_stuff_changed_cb (self, "local_network_entry");
 	hook_stuff_changed_cb (self, "remote_network_entry");
 	hook_stuff_changed_cb (self, "narrowing_checkbutton");
 	hook_stuff_changed_cb (self, "fragmentation_combo");
@@ -492,6 +496,8 @@ init_editor_plugin (LibreswanEditor *self,
 	hook_stuff_changed_cb (self, "dpd_action_combo");
 	hook_stuff_changed_cb (self, "ipsec_interface_entry");
 	hook_stuff_changed_cb (self, "authby_entry");
+	hook_stuff_changed_cb (self, "disable_modecfgclient_checkbutton");
+	hook_stuff_changed_cb (self, "remote_cert_entry");
 
 	priv->advanced_dialog = GTK_WIDGET (gtk_builder_get_object (priv->builder, "libreswan-advanced-dialog"));
 	g_return_val_if_fail (priv->advanced_dialog != NULL, FALSE);
@@ -561,6 +567,16 @@ update_adv_settings (LibreswanEditor *self, NMSettingVpn *s_vpn)
 	else
 		nm_setting_vpn_remove_data_item (s_vpn, NM_LIBRESWAN_KEY_DOMAIN);
 
+	/* Local Network */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+	                                             "local_network_entry"));
+	str = gtk_editable_get_text (GTK_EDITABLE (widget));
+	if (str && *str)
+		nm_setting_vpn_add_data_item (s_vpn, NM_LIBRESWAN_KEY_LOCALNETWORK, str);
+	else
+		nm_setting_vpn_remove_data_item (s_vpn, NM_LIBRESWAN_KEY_LOCALNETWORK);
+
+
 	/* Remote Network */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
 	                                             "remote_network_entry"));
@@ -658,6 +674,21 @@ update_adv_settings (LibreswanEditor *self, NMSettingVpn *s_vpn)
 		nm_setting_vpn_add_data_item (s_vpn, NM_LIBRESWAN_KEY_AUTHBY, str);
 	else
 		nm_setting_vpn_remove_data_item (s_vpn, NM_LIBRESWAN_KEY_AUTHBY);
+
+	/* Disable Mode Config client */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "disable_modecfgclient_checkbutton"));
+	if (gtk_check_button_get_active (GTK_CHECK_BUTTON (widget)))
+		nm_setting_vpn_add_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTMODECFGCLIENT, "no");
+	else
+		nm_setting_vpn_remove_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTMODECFGCLIENT);
+
+	/* Remote certificate */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "remote_cert_entry"));
+	str = gtk_editable_get_text (GTK_EDITABLE (widget));
+	if (str && *str)
+		nm_setting_vpn_add_data_item (s_vpn, NM_LIBRESWAN_KEY_RIGHTCERT, str);
+	else
+		nm_setting_vpn_remove_data_item (s_vpn, NM_LIBRESWAN_KEY_RIGHTCERT);
 }
 
 static gboolean
